/*
 * $Id: kbd.c,v 1.3 2000/06/19 19:29:32 harald Exp $
 *
 * Sharp Mobilon HC-4500G keyboard driver
 *
 * Copyright (C) 1999 Harald Koerfgen
 * 
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 */

#include <linux/sched.h>
#include <linux/tty.h>
#include <linux/mm.h>
#include <linux/kbd_kern.h>
#include <linux/errno.h>
#include <linux/init.h>
#include <linux/kbd_ll.h>
#include <asm/keyboard.h>
#include <asm/r39xx.h>
#include <asm/bootinfo.h>

#define CLEAR_SCANLINES {unsigned short dummy; dummy = *(volatile unsigned short *)KSEG3;}

volatile unsigned short * scanline[] = {
	(volatile unsigned short *)KSEG3ADDR(0x002),
	(volatile unsigned short *)KSEG3ADDR(0x004),
	(volatile unsigned short *)KSEG3ADDR(0x008),
	(volatile unsigned short *)KSEG3ADDR(0x010),
	(volatile unsigned short *)KSEG3ADDR(0x020),
	(volatile unsigned short *)KSEG3ADDR(0x040),
	(volatile unsigned short *)KSEG3ADDR(0x080),
	(volatile unsigned short *)KSEG3ADDR(0x100)
};

void kbd_scan(void);
static void kbd_timer_handler(unsigned long);

static struct timer_list kbd_timer;

static void kbd_timer_handler(unsigned long dev_addr)
{
      kbd_scan();
      mod_timer(&kbd_timer, jiffies + HZ / 20);
}

void kbd_leds(unsigned char leds)
{
	return;
}

int kbd_setkeycode(unsigned int scancode, unsigned int keycode)
{
	return -EINVAL;
}

int kbd_getkeycode(unsigned int scancode)
{
	return -EINVAL;
}

int kbd_translate(unsigned char scancode, unsigned char *keycode,
	char raw_mode)
{
	*keycode = scancode;
	return 1;
}

char kbd_unexpected_up(unsigned char keycode)
{
	printk(__FUNCTION__": unexpected up, keycode 0x%x\n", keycode);
	return 0x80;
}

void kbd_scan(void)
{
	int i, j;
	unsigned short scan;
	unsigned short diff, bitmask;
	static unsigned short last_scan[8] = {0, 0, 0, 0, 0, 0, 0, 0};
	unsigned char hi_nibble;

	CLEAR_SCANLINES;
	udelay(3);

	for (i = 0; i < 8; i++) {
		scan = *scanline[i];		
		udelay(3);
		CLEAR_SCANLINES;
		udelay(3);

		if (diff = (scan ^ last_scan[i])) {
			hi_nibble = i << 4;
			for (j = 0; j < 16; j++) {
				bitmask = 1 << j;
				if (diff & bitmask) {
					handle_scancode (hi_nibble | j, scan & bitmask);
//					printk("key %d %s\n", hi_nibble | j, (scan & bitmask) ? "down" : "up");
				}
			}
			last_scan[i] = scan;
		}
	}
}

void __init kbd_init_hw(void)
{
	switch (mips_machtype) {
	case MACH_R39XX_SHARP_MOBILON:
		printk("Keyboard driver version 0.2 for Sharp Mobilon\n");

		init_timer(&kbd_timer);
		kbd_timer.expires = jiffies + HZ / 20;
		kbd_timer.function = kbd_timer_handler;
		add_timer(&kbd_timer);
		break;
	case MACH_R39XX_COMPAQ:
		printk("Keyboard driver version 0.2 for Compaq C-Series\n");

		init_timer(&kbd_timer);
		kbd_timer.expires = jiffies + HZ / 20;
		kbd_timer.function = kbd_timer_handler;
		add_timer(&kbd_timer);
		break;
	default:
		printk("Sorry, your machine doesn't have a keyboard\n");
		break;
	}
}
