/* $Id: power.c,v 1.1 2000/07/05 15:27:02 nop Exp $
 *
 * power - Routines for power management on r39xx
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 2000 Jim Pick <jim@jimpick.com>
 *
 */

#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/signal.h>
#include <linux/errno.h>
#include <linux/mm.h>
#include <linux/poll.h>
#include <linux/miscdevice.h>
#include <linux/random.h>
#include <linux/init.h>
#include <asm/mipsregs.h>
#include <asm/system.h>
#include <asm/r39xx.h>
#include <asm/betty.h>
#include "sib.h"

static int sample_counter;

#ifndef CONFIG_VTECH_HELIO
#error too platform-specific right now, fix later, patches wanted
#endif

#if 0
static void get_battery_voltage_mode(void)
{
	sib_write_check(BETTY_REG_ADC_CTRL,
			(BETTY_ADC_INPUT_AD0 <<
			 BETTY_ADC_INPUT_SHIFT) |
			BETTY_ADC_SYNC_ENA |
			BETTY_VREFBYP_CON |
			BETTY_ADC_ENA);

	sib_write_check(BETTY_REG_ADC_CTRL,
			(BETTY_ADC_INPUT_AD0 <<
			 BETTY_ADC_INPUT_SHIFT) |
			BETTY_ADC_SYNC_ENA |
			BETTY_VREFBYP_CON |
			BETTY_ADC_ENA |
			BETTY_ADC_START);
}

static void end_battery_voltage_mode(void)
{
	sib_write_check(BETTY_REG_ADC_CTRL,
			(BETTY_ADC_INPUT_AD0 <<
			 BETTY_ADC_INPUT_SHIFT) |
			BETTY_ADC_SYNC_ENA |
			BETTY_VREFBYP_CON);
}

static int get_battery_voltage_state = 0;

void get_battery_voltage(void)
{
	unsigned long value;

	switch( get_battery_voltage_state ) {

	case 0:
		/* Now, get battery voltage */

		get_battery_voltage_mode();

		sample_counter = 0;

		get_battery_voltage_state = 1;
		sib_read_check(BETTY_REG_ADC_DATA, get_battery_voltage);
	     
		break;

	case 1:

		/* Some voltage data was recieved from ADC */ 

		value = SIBSubFrame0Status;
		
		if (! (value & BETTY_ADC_DAT_VAL)) {
			if (++sample_counter < 60)
				sib_read_check(BETTY_REG_ADC_DATA,
					       get_battery_voltage);
			else {
				/* Restore SIB state */

				end_battery_voltage_mode();

				get_battery_voltage_state = 0;
			}
			break;
		}

		/* If valid, then fall through */

		printk("V: %d\n", 
		       (value & BETTY_ADC_DATA_MASK) >> BETTY_ADC_DATA_SHIFT);

		end_battery_voltage_mode();

		break;

	}
}
#endif

#ifdef CONFIG_VTECH_HELIO

static void set_helio_speed_75mhz(void)
{
    /* Set processor speed to 75MHz by zeroing reduced frequency
       bits in r39xx config register, and unsetting SlowBus */

/*
    printk("CP0_CONF:      0x%08x\n",  read_32bit_cp0_register(CP0_CONF));
    printk("PowerControl:  0x%08x\n", PowerControl);
*/

    set_cp0_conf(0xffff,
		 (1 << CONFB_ICE) | (1 << CONFB_DCE));
    PowerControl &= ~PWR_SLOWBUS; 
}


static void set_helio_speed_4mhz(void)
{
    /* Set processor speed to 4MHz? by setting reduced frequency
       bits in r39xx config register, and setting SlowBus */

    set_cp0_conf(0xffff, (1 << CONFB_ICE) | (1 << CONFB_DCE) |
		 (3 << CONFB_RF));
    PowerControl |= PWR_SLOWBUS; 
}

void pmtemp_max_doze(void)
{
        MFIOOutput &= ~MFIO_PIN_BACKLIGHT;
        turn_off_helio_lcd();
        turn_off_helio_serial();
        MFIOOutput &= ~MFIO_PIN_BACKLIGHT;
        set_helio_speed_4mhz();
}

void pmtemp_resume(void) 
{
        turn_on_helio_lcd();
        turn_on_helio_serial();
        set_helio_speed_75mhz();
}

#else

void pmtemp_max_doze(void)
{
}

void pmtemp_resume(void)
{
}

#endif

