/*
 * linux/arch/mips/vr41xx/nec_osprey.c
 *
 * Platform support for NEC Osprey.
 *
 * Copyright (C) 2000 Bradley D. LaRonde
 * Copyright (C) 2000 Michael Klar
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 */

#include <asm/io.h>
#include <asm/vr41xx.h>

static void display_setup(void)
{
	// Use GP12-15 for color LCD FPD[4-7] output.
	*VR41XX_GPMD1REG |= 0xff00;

	// Set up programmable chip select 0 for LCD.
	#define LCD_BLIGHT_CONT_PORT 0x03ffff00
	#define LBLC_PHYSADR_START (LCD_BLIGHT_CONT_PORT + VR41XX_ISA_IO)
	#define LBLC_PHYSADR_STOP LBLC_PHYSADR_START

	*VR41XX_PCS0STRA = LBLC_PHYSADR_START & 0xffff;
	*VR41XX_PCS0STPA = LBLC_PHYSADR_STOP & 0xffff;
	*VR41XX_PCS0HIA =  (LBLC_PHYSADR_START >> 16) & 0x0fff;

	// Set up programmable chip select mode.
	*VR41XX_PCSMODE = 0x2;

	// Set gclk to Hpck ratio.
	*VR41XX_LCDCFGREG1 = 0x0707;
}

int display_backlight(int n)
{
	char c = inb(LCD_BLIGHT_CONT_PORT);
	if(n == 0)
		// Turn the backlight off.
		outb(c & ~0x30, LCD_BLIGHT_CONT_PORT);
	else
		// Turn the backlight on.
		outb(c | 0x30, LCD_BLIGHT_CONT_PORT);
	return 0;
}

int display_contrast(int n)
{
	char c = inb(LCD_BLIGHT_CONT_PORT);
	outb((c & 0xf0) | n, LCD_BLIGHT_CONT_PORT);
	return 0;
}

void platdep_setup(void)
{
	display_setup();
	display_contrast(6);
	display_backlight(1);
}
