/*
    linux/drivers/pcmcia/pccard.h

    Copyright (C) 2000 by Yoichi Yuasa <yuasa@hh.iij4u.or.jp>

    This is distributed under the GNU GENERAL PUBLIC LICENSE (GPL)
    Version 2 (June 1991). See the "COPYING" file distributed with this
    software for more info.
*/

#define MAX_SOCKS	8
#define MAX_BINDINGS    4
#define MAX_MODULES     4

typedef struct func_ident_t {
    u_char		funcid;
} func_ident_t;

typedef struct manfid_ident_t {
    u_short		manf;
    u_short		card;
} manfid_ident_t;

typedef struct vers_ident_t {
    int			ns;
    char		*pi[4];
} vers_ident_t;

typedef struct tuple_ident_t {
    cisdata_t		code;
    long		ofs;
    char		*info;
} tuple_ident_t;

typedef struct device_info_t {
    dev_info_t		dev_info;
    int			needs_mtd;
    int			modules;
    char		*module[MAX_MODULES];
    char		*opts[MAX_MODULES];
    char		*class;
    int			refs;
    struct device_info_t *next;
} device_info_t;

typedef struct card_info_t {
    int			bindings;
    char                *device[MAX_BINDINGS];
    int                 dev_fn[MAX_BINDINGS];
    int			refs;
} card_info_t;

typedef struct vers_info_t {
    char		*name;
    vers_ident_t	vers;
} vers_info_t;

typedef struct manfid_info_t {
    char		*name;
    manfid_ident_t	manfid;
} manfid_info_t;

typedef struct tuple_info_t {
    char		*name;
    tuple_ident_t	tuple;
} tuple_info_t;

typedef struct cardcfg_info_t {
    enum {
	VERS_1_IDENT=1, MANFID_IDENT, TUPLE_IDENT, FUNC_IDENT,
	BLANK_IDENT, PCI_IDENT
    } ident_type;
    void		*id;
    size_t		size;
    card_info_t         *card;
} cardcfg_info_t;

typedef struct funcid_info_t {
    char		*name;
    func_ident_t	func;
    card_info_t         *card;
} funcid_info_t;

#define MULTI_FUNCTION	0x00
#define MEMORY		0x01
#define SERIAL_PORT	0x02
#define PARALLEL_PORT	0x03
#define FIXED_DISK	0x04
#define VIDEO		0x05
#define NETWORK_ADAPTER	0x06
#define AIMS		0x07
#define SCSI		0x08
#define SECURITY	0x09
#define INSTRUMENT	0x0a
#define VENDER_DEF	0xfe
#define NOT_USE		0xff

/*====================================================================*/
/* PC card driver bindings */

card_info_t ide_cs =
    { 1, {"ide_cs", NULL, NULL, NULL}, {0, 0, 0, 0}, 0 };

card_info_t c574_cs =
    { 1, {"3c574_cs", NULL, NULL, NULL}, {0, 0, 0, 0}, 0 };

card_info_t c589_cs =
    { 1, {"3c589_cs", NULL, NULL, NULL}, {0, 0, 0, 0}, 0 };

card_info_t aironet4500_cs =
    { 1, {"aironet4500_cs", NULL, NULL, NULL}, {0, 0, 0, 0}, 0 };

card_info_t com20020_cs =
    { 1, {"com20020_cs", NULL, NULL, NULL}, {0, 0, 0, 0}, 0 };

card_info_t fmvj18x_cs =
    { 1, {"fmvj18x_cs", NULL, NULL, NULL}, {0, 0, 0, 0}, 0 };

card_info_t netwave_cs =
    { 1, {"netwave_cs", NULL, NULL, NULL}, {0, 0, 0, 0}, 0 };

card_info_t nmclan_cs =
    { 1, {"nmclan_cs", NULL, NULL, NULL}, {0, 0, 0, 0}, 0 };

card_info_t pcnet_cs =
    { 1, {"pcnet_cs", NULL, NULL, NULL}, {0, 0, 0, 0}, 0 };

card_info_t ray_cs =
    { 1, {"ray_cs", NULL, NULL, NULL}, {0, 0, 0, 0}, 0 };

card_info_t smc91c92_cs =
    { 1, {"smc91c92_cs", NULL, NULL, NULL}, {0, 0, 0, 0}, 0 };

card_info_t wavelan_cs =
    { 1, {"wavelan_cs", NULL, NULL, NULL}, {0, 0, 0, 0}, 0 };

card_info_t xirc2ps_cs =
    { 1, {"xirc2ps_cs", NULL, NULL, NULL}, {0, 0, 0, 0}, 0 };

card_info_t serial_cs =
    { 1, {"serial_cs", NULL, NULL, NULL}, {0, 0, 0, 0}, 0 };

/*====================================================================*/

#define _pi(s)        (char *)s

/* Linked list of card definitions */
/* ide_cs */
vers_info_t ide_cs_vers[] = {
    { "ACE DoubleFlash 20MB",
      {2, {_pi("IBM"), _pi("IBM17JSSFP20"), NULL, NULL}}},
    { "Argosy EIDE CD-ROM",
      {2, {_pi("ARGOSY"), _pi("CD-ROM"), NULL, NULL}}},
    { "Caravelle CD-36N/CD-36E CD-ROM",
      {3, {_pi("Caravelle"), _pi("PSC-IDE "), _pi("PSC000"), NULL}}},
    { "CNF CARDport CD-ROM",
      {2, {_pi("CNF CD-M"), _pi("CD-ROM"), NULL, NULL}}},
    { "Creative Technology CD-ROM",
      {2, {_pi("Creative Technology Ltd."), _pi("PCMCIA CD-ROM Interface Card"), NULL, NULL}}},
    { "Digital Mobile Media CD-ROM",
      {2, {_pi("Digital Equipment Corporation."), _pi("Digital Mobile Media CD-ROM"), NULL, NULL}}},
    { "EXP CD940 CD-ROM",
      {2, {_pi("EXP   "), _pi("CD-ROM"), NULL, NULL}}},
    { "EXP Traveler 620 CD-ROM",
      {3, {_pi("EXP"), _pi("CD+GAME"), _pi("C1"), NULL}}},
    { "Freecom IQ-drive",
      {2, {_pi("FREECOM"), _pi("PCCARD-IDE"), NULL, NULL}}},
    { "Generic ATAPI CD-ROM",
      {2, {_pi("PCMCIA"), _pi("CD-ROM"), NULL, NULL}}},
    { "IO DATA CDP-TX/PCIDE",
      {2, {_pi("IO DATA"), _pi("PCIDE"), NULL, NULL}}},
    { "IO DATA PCIDE II",
      {2, {_pi("IO DATA"), _pi("PCIDEII"), NULL, NULL}}},
    { "IO DATA CBIDE",
      {2, {_pi("WIT"), _pi("IDE16"), NULL, NULL}}},
    { "MCD601p CD-ROM",
      {2, {_pi("CDROM"), _pi("IDE"), _pi("MCD-601p"), NULL}}},
    { "Ninja ATA",
      {2, {_pi(" "), _pi("NinjaATA-"), NULL, NULL}}},
    { "Simple Tech Compact Flash",
      {1, {_pi("STI Flash"), NULL, NULL, NULL}}}
};

manfid_info_t ide_cs_manfid[] = {
    { "IBM Max Portable CD-ROM",
      {0x00a4, 0x002d}}
};

#define IDE_CS_VERS_SIZE	(sizeof(ide_cs_vers)/sizeof(ide_cs_vers[0]))
#define IDE_CS_MANFID_SIZE	(sizeof(ide_cs_manfid)/sizeof(ide_cs_manfid[0]))
#define IDE_CS_TUPLE_SIZE	0

/*====================================================================*/

/* 3c574_cs */
manfid_info_t c574_cs_manfid[] = {
    { "3Com 572/574 Fast Ethernet",
      {0x0101, 0x0574}}
};

#define C574_CS_VERS_SIZE	0
#define C574_CS_MANFID_SIZE	sizeof(c574_cs_manfid)/sizeof(c574_cs_manfid[0])
#define C574_CS_TUPLE_SIZE	0

/*====================================================================*/

/* 3c589_cs */
vers_info_t c589_cs_vers[] = {
    { "Farallon ENet",
      {2, {_pi("Farallon"), _pi("ENet"), NULL, NULL}}}
};

manfid_info_t c589_cs_manfid[] = {
    { "3Com 589 Ethernet",
      {0x0101, 0x0589}}
};

#define C589_CS_VERS_SIZE	sizeof(c589_cs_vers)/sizeof(c589_cs_vers[0])
#define C589_CS_MANFID_SIZE	sizeof(c574_cs_manfid)/sizeof(c574_cs_manfid[0])
#define C589_CS_TUPLE_SIZE	0

/*====================================================================*/

/* aironet4500_cs */
manfid_info_t aironet4500_cs_manfid[] = {
    { "Aironet PC4500",
      {0x015f, 0x0005}},
    { "Aironet PC4800",
      {0x015f, 0x0007}}
};

#define AIRONET4500_CS_VERS_SIZE	0
#define AIRONET4500_CS_MANFID_SIZE	sizeof(aironet4500_cs_manfid)/sizeof(aironet4500_cs_manfid[0])
#define AIRONET4500_CS_TUPLE_SIZE	0

/*====================================================================*/

/* com20020_cs */
#define COM20020_CS_VERS_SIZE	0
#define COM20020_CS_MANFID_SIZE	0
#define COM20020_CS_TUPLE_SIZE	0

/*====================================================================*/

/* fmvj18x_cs */
vers_info_t fmvj18x_cs_vers[] = {
    { "CONTEC Ethernet",
      {2, {_pi("CONTEC Co.,Ltd."), NULL, NULL, NULL}}},
    { "Eiger Labs EPX-10BT Ethernet",
      {2, {_pi("Eiger Labs,Inc"), _pi("EPX-10BT PC Card Ethernet 10BT"), NULL, NULL}}},
    { "Eiger Labs EPX-ET 10BT Ethernet",
      {2, {_pi("Eiger labs,Inc."), _pi("EPX-10BT PC Card Ethernet 10BT"), NULL, NULL}}},
    { "Fujitsu FMV-J181 Ethernet",
      {1, {_pi("PCMCIA MBH10302"), NULL, NULL, NULL}}},
    { "Fujitsu FMV-1080 Ethernet",
      {2, {_pi("FUJITSU"), _pi("MBH10308"), NULL, NULL}}},
    { "Fujitsu FMV-J182 Ethernet",
      {2, {_pi("FUJITSU"), _pi("LAN Card(FMV-J182)"), NULL, NULL}}},
    { "Fujitsu Towa LA501 Ethernet",
      {2, {_pi("FUJITSU TOWA"), _pi("LA501"), NULL, NULL}}},
    { "Hitachi HT-4840-11 Ethernet",
      {2, {_pi("HITACHI"), _pi("HT-4840-11"), NULL, NULL}}},
    { "NextCom NC5310 Ethernet",
      {2, {_pi("NextComK.K."), _pi("NC5310 Ver1.0        "), NULL, NULL}}},
    { "Novell Eagle 200T Ethernet",
      {2, {_pi("EAGLE Technology"), _pi("NE200 ETHERNET LAN MBH10302 04"), NULL, NULL}}},
    { "RATOC Ethernet",
      {1, {_pi("PCMCIA LAN MBH10304  ES"), NULL, NULL, NULL}}},
    { "RATOC REX-R280 Ethernet",
      {2, {_pi("RATOC System Inc."), _pi("10BASE_T CARD R280"), NULL, NULL}}},
    { "TDK LAC-CD02x Ethernet",
      {2, {_pi("TDK"), _pi("LAC-CD02x"), NULL, NULL}}}
};

manfid_info_t fmvj18x_cs_manfid[] = {
    { "Eagle NE200 Ethernet",
      {0x0004, 0x0004}}
};

#define FMVJ18X_CS_VERS_SIZE	sizeof(fmvj18x_cs_vers)/sizeof(fmvj18x_cs_vers[0])
#define FMVJ18X_CS_MANFID_SIZE	sizeof(fmvj18x_cs_manfid)/sizeof(fmvj18x_cs_manfid[0])
#define FMVJ18X_CS_TUPLE_SIZE	0

/*====================================================================*/

/* netwave_cs */
vers_info_t netwave_cs_vers[] = {
    { "Xircom CreditCard Netwave",
      {2, {_pi("Xircom"), _pi("CreditCard Netwave"), NULL, NULL}}},
};

#define NETWAVE_CS_VERS_SIZE	sizeof(netwave_cs_vers)/sizeof(netwave_cs_vers[0])
#define NETWAVE_CS_MANFID_SIZE	0
#define NETWAVE_CS_TUPLE_SIZE	0

/*====================================================================*/

vers_info_t nmclan_cs_vers[] = {
    { "New Media Ethernet",
      {2, {_pi("New Media Corporation"), _pi("Ethernet"), NULL, NULL}}}
};

#define NMCLAN_CS_VERS_SIZE	sizeof(nmclan_cs_vers)/sizeof(nmclan_cs_vers[0])
#define NMCLAN_CS_MANFID_SIZE	0
#define NMCLAN_CS_TUPLE_SIZE	0

/*====================================================================*/

/* pcnet_cs */
vers_info_t pcnet_cs_vers[] = {
    { "2412LAN Ethernet",
      {1, {_pi("2412LAN"), NULL, NULL, NULL}}},
    { "Accton EN2212 Ethernet",
      {2, {_pi("ACCTON"), _pi("EN2212"), NULL, NULL}}},
    { "Accton EN2216 Ethernet",
      {2, {_pi("ACCTON"), _pi("EN2216-PCMCIA-ETHERNET"), NULL, NULL}}},
    { "Accton UE2212 Ethernet",
      {2, {_pi("PCMCIA"), _pi("UE2212"), NULL, NULL}}},
    { "Accton UE2216 Ethernet",
      {3, {_pi("PCMCIA"), _pi("PCMCIA-ETHERNET-CARD"), _pi("UE2216"), NULL}}},
    { "AmbiCom AMB8002 Ethernet",
      {2, {_pi("AmbiCom Inc"), _pi("AMB8002"), NULL, NULL}}},
    { "AmbiCom AMB8002T Ethernet",
      {2, {_pi("AmbiCom Inc"), _pi("AMB8002T"), NULL, NULL}}},
    { "AmbiCom AMB8010 Ethernet",
      {2, {_pi("AmbiCom Inc"), _pi("AMB8010"), NULL, NULL}}},
    { "AnyCom ECO Ethernet",
      {2, {_pi("AnyCom"), _pi("ECO Ethernet"), NULL, NULL}}},
    { "Argosy EN210 Ethernet",
      {2, {_pi("PCMCIA LAN"), _pi("Ethernet"), NULL, NULL}}},
    { "Asante FriendlyNet Ethernet",
      {2, {_pi("ASANTE"), _pi("FriendlyNet PC Card"), NULL, NULL}}},
    { "Billionton LNT-10TB",
      {2, {_pi("Billionton"), _pi("LNT-10TB"), NULL, NULL}}},
    { "Billionton LNT-10TN",
      {2, {_pi("PCMCIA"), _pi("LNT-10TN"), NULL, NULL}}},
    { "California Access Ethernet",
      {2, {_pi("PCMCIAs"), _pi("ComboCard"), NULL, NULL}}},
    { "CeLAN Ethernet",
      {2, {_pi("PCMCIA"), _pi("ETHERNET V1.0"), NULL, NULL}}},
    { "CentreCOM LA-PCM V2 Ethernet",
      {2, {_pi("Allied Telesis, K.K."), _pi("CentreCOM LA-PCM_V2"), NULL, NULL}}},
    { "CNet CN30BC Ethernet",
      {3, {_pi("CNet  "), _pi("CN30BC"), _pi("ETHERNET"), NULL}}},
    { "CNet CN40BC Ethernet",
      {2, {_pi("CNet"), _pi("CN40BC Ethernet"), NULL, NULL}}},
    { "COMPU-SHACK BASEline Ethernet",
      {2, {_pi("COMPU-SHACK"), _pi("BASEline PCMCIA 10 MBit Ethernetadapter"), NULL, NULL}}},
    { "Corega PCC-T Ethernet",
      {2, {_pi("corega K.K."), _pi("corega Ether PCC-T"), NULL, NULL}}},
    { "Corega PCC-T Ethernet",
      {2, {_pi("corega K.K."), _pi("corega EtherII PCC-T"), NULL, NULL}}},
    { "Corega PCC-T Ethernet",
      {2, {_pi("corega K.K."), _pi("Ethernet LAN Card"), NULL, NULL}}},
    { "Corega PCM-T Ethernet",
      {2, {_pi("Corega,K.K."), _pi("Ethernet LAN Card"), NULL, NULL}}},
    { "CyQ've 10baseT Ethernet",
      {1, {_pi("CyQ've 10 Base-T LAN CARD"), NULL, NULL, NULL}}},
    { "CyQ've ELA-010 Ethernet",
      {2, {_pi("CyQ've"), _pi("ELA-010"), NULL, NULL}}},
    { "Danpex EN-6200P2 Ethernet",
      {2, {_pi("*"), _pi("EN-6200P2"), NULL, NULL}}},
    { "DataTrek NetCard Ethernet",
      {2, {_pi("DataTrek."), _pi("NetCard "), NULL, NULL}}},
    { "Dayna Commnunications CommuniCard E Ethernet",
      {2, {_pi("Dayna Communications, Inc."), _pi("CommuniCard E"), NULL, NULL}}},
    { "Digital DEPCM-BA Ethernet",
      {2, {_pi("DIGITAL"), _pi("DEPCM-XX"), NULL, NULL}}},
    { "Digital PCP78-AC Ethernet",
      {3, {_pi("Digital"), _pi("Ethernet"), _pi("Adapter"), NULL}}},
    { "D-Link DE-650 Ethernet",
      {2, {_pi("D-Link"), _pi("DE-650"), NULL, NULL}}},
    { "D-Link DFE-650 Ethernet",
      {2, {_pi("D-Link"), _pi("DFE-650"), NULL, NULL}}},
    { "D-Link DE-660 Ethernet",
      {2, {_pi("D-Link"), _pi("DE-660"), NULL, NULL}}},
    { "DynaLink L10C Ethernet",
      {2, {_pi("DYNALINK"), _pi("L10C"), NULL, NULL}}},
    { "E2000 NE2000-Compatible Ethernet",
      {2, {_pi("Ethernet Adapter"), _pi("E2000 PCMCIA Ethernet"), NULL, NULL}}},
    { "Edimax Ethernet Combo",
      {2, {_pi("Edimax Technology Inc."), _pi("PCMCIA"), _pi("Ethernet Card"), NULL}}},
    { "EFA 207 Ethernet",
      {3, {_pi("EFA   "), _pi("EFA207"), _pi("ETHERNET"), NULL}}},
    { "Eiger Labs EPX-ET10T2 Ethernet",
      {2, {_pi("EIGER Labs Inc."), _pi("Ethernet Combo card"), NULL, NULL}}},
    { "Eiger Labs EPX-ET10BT Ethernet",
      {2, {_pi("EIGER Labs Inc."), _pi("Ethernet 10BaseT card"), NULL, NULL}}},
    { "ELECOM Laneed LD-CDF",
      {2, {_pi("Laneed"), _pi("LD-CDF"), NULL, NULL}}},
    { "EZLink 4109 Ethernet",
      {2, {_pi("PCMCIA"), _pi("Ethernet Card"), NULL, NULL}}},
    { "EP-210 Ethernet",
      {2, {_pi("Ethernet PCMCIA adapter"), _pi("EP-210"), NULL, NULL}}},
    { "EP-210 Ethernet",
      {1, {_pi("EP-210 PCMCIA LAN CARD."), NULL, NULL, NULL}}},
    { "EP-220 Ethernet",
      {2, {_pi("PCMCIA"), _pi("ETHERNET"), NULL, NULL}}},
    { "Epson EEN10B Ethernet",
      {2, {_pi("Seiko Epson Corp."), _pi("Ethernet"), NULL, NULL}}},
    { "Grey Cell GCS2000 Ethernet",
      {2, {_pi("Grey Cell"), _pi("GCS2000"), NULL, NULL}}},
    { "Grey Cell GCS2220 Ethernet",
      {2, {_pi("Grey Cell"), _pi("GCS2220"), NULL, NULL}}},
    { "GVC NIC-2000P Ethernet",
      {2, {_pi("GVC"), _pi("NIC-2000p"), NULL, NULL}}},
    { "GVC NIC-2000P Ethernet",
      {2, {_pi("PCMCIA"), _pi("Ethernet Combo card"), NULL, NULL}}},
    { "Hypertec EP401 Ethernet",
      {2, {_pi("Hypertec"), _pi("*"), _pi("EP401"), NULL}}},
    { "IBM Credit Card Ethernet",
      {2, {_pi("IBM Corp."), _pi("Ethernet"), NULL, NULL}}},
    { "IC-Card Ethernet",
      {2, {_pi("IC-CARD"), _pi("IC-CARD"), NULL, NULL}}},
    { "IC-Card Plus Ethernet",
      {2, {_pi("IC-CARD+"), _pi("IC-CARD+"), NULL, NULL}}},
    { "Kansai KLA-PCM/T Ethernet",
      {2, {_pi("KANSAI ELECTRIC CO.,LTD"), _pi("KLA-PCM/T"), NULL, NULL}}},
    { "Katron PE-520 Ethernet",
      {2, {_pi("KCI"), _pi("PE520 PCMCIA Ethernet Adapter"), NULL, NULL}}},
    { "KingMax Technology Ethernet",
      {3, {_pi("KingMax Technology Inc."), _pi("EN10-T2"),
           _pi("PCMCIA Ethernet Card"), NULL}}},
    { "KingMax Technology Ethernet",
      {3, {_pi("KingMax Technology Inc."), _pi("*"),
           _pi("Ethernet Card"), NULL}}},
    { "KingMax Technology Ethernet",
      {2, {_pi("KINGMAX"), _pi("EN10T2T"), NULL, NULL}}},
    { "Kingston KNE-PCM/x Ethernet",
      {2, {_pi("Kingston Technology Corp."),
           _pi("EtheRx PC Card Ethernet Adapter"), NULL, NULL}}},
    { "Kingston KNE-PC2 Ethernet",
      {2, {_pi("Kingston"), _pi("KNE-PC2"), NULL, NULL}}},
    { "KTI PE-520 Plus Ethernet",
      {2, {_pi("KTI"), _pi("PE520 PLUS"), NULL, NULL}}},
    { "Linksys Ethernet",
      {2, {_pi("LINKSYS"), _pi("E-CARD"), NULL, NULL}}},
    { "Linksys Ethernet",
      {2, {_pi("E-CARD"), _pi("E-CARD"), NULL, NULL}}},
    { "Linksys Ethernet",
      {2, {_pi("Linksys"),
           _pi("Combo PCMCIA EthernetCard (EC2T)"), NULL, NULL}}},
    { "Longshine ShineNet LCS-8534 Ethernet",
      {2, {_pi("LONGSHINE"), _pi("PCMCIA Ethernet Card"), NULL, NULL}}},
    { "Longshine ShineNet LCS-8534 Ethernet",
      {3, {_pi("LONGSHINE"), _pi("*"), _pi("EP401"), NULL}}},
    { "Logitec LPM-LN10BA Ethernet",
      {3, {_pi("Cardwell"), _pi("PCMCIA"), _pi("ETHERNET"), NULL}}},
    { "Logitec LPM-LN20T Ethernet",
      {2, {_pi("Logitec"), _pi("LPM-LN20T"), NULL, NULL}}},
    { "MACNICA Ethernet",
      {2, {_pi("MACNICA"), _pi("ME1-JEIDA"), NULL, NULL}}},
    { "Maxtech PCN2000 Ethernet",
      {2, {_pi("MAXTECH"), _pi("PCN2000"), NULL, NULL}}},
    { "Microdyne NE4200 Ethernet",
      {2, {_pi("Microdyne"), _pi("NE4200"), NULL, NULL}}},
    { "Midori LT-PCMT Ethernet",
      {2, {_pi("MIDORI ELEC."), _pi("LT-PCMT"), NULL, NULL}}},
    { "National Semiconductor InfoMover 4100 Ethernet",
      {2, {_pi("National Semiconductor"), _pi("InfoMover 4100"), NULL, NULL}}},
    { "National Semiconductor InfoMover NE4100 Ethernet",
      {2, {_pi("National Semiconductor"), _pi("InfoMover NE4100"), NULL, NULL}}},
    { "NEC PC-9801N-J12 Ethernet",
      {2, {_pi("NEC"), _pi("PC-9801N-J12"), NULL, NULL}}},
    { "NE2000 Compatible Ethernet",
      {2, {_pi("PCMCIA"), _pi("Ethernet"), NULL, NULL}}},
    { "NE2000 Compatible Ethernet",
      {2, {_pi("PCMCIA"), _pi("Ethernet 10BaseT Card"), NULL, NULL}}},
    { "NE2000 Compatible Ethernet",
      {3, {_pi("NE2000 Compatible"), _pi("PCMCIA"), _pi("Ethernet Card"), NULL}}},
    { "NE2000 Compatible Ethernet",
      {2, {_pi("Ethernet"), _pi("Adapter"), NULL, NULL}}},
    { "NDC Instant-Link Ethernet",
      {2, {_pi("NDC"), _pi("Ethernet"), NULL, NULL}}},
    { "IO DATA PCLA/T Ethernet",
      {3, {_pi("I-O DATA"), _pi("PCLA"), _pi("ETHERNET"), NULL}}},
    { "IO DATA PCLA/TE Ethernet",
      {3, {_pi("IO DATA"), _pi("PCLATE"), _pi("ETHERNET"), NULL}}},
    { "Panasonic CF-VEL211 Ethernet",
      {2, {_pi("Matsushita Electric Industrial Co.,LTD."),
           _pi("CF-VEL211"), NULL, NULL}}},
    { "PreMax PE-200 Ethernet",
      {2, {_pi("PMX   "), _pi("PE-200"), NULL, NULL}}},
    { "Psion Gold Card Ethernet",
      {2, {_pi("Psion Dacom"), _pi("Gold Card Ethernet"), NULL, NULL}}},
    { "Relia RE2408T Ethernet",
      {2, {_pi("PCMCIA LAN"), _pi("Ethernet"), NULL, NULL}}},
    { "Reliasys 2400A Ethernet",
      {2, {_pi("=RELIA=="), _pi("Ethernet"), NULL, NULL}}},
    { "RPTI EP400 Ethernet",
      {2, {_pi("RPTI LTD."), _pi("EP400"), NULL, NULL}}},
    { "RPTI EP400 Ethernet",
      {2, {_pi("RPTI"), _pi("EP400 Ethernet NE2000 Compatible"), NULL, NULL}}},
    { "RPTI EP401 Ethernet",
      {2, {_pi("RPTI"), _pi("EP401 Ethernet NE2000 Compatible"), NULL, NULL}}},
    { "RPTI 1625B Ethernet",
      {2, {_pi("RP"), _pi("1625B Ethernet NE2000 Compatible"), NULL, NULL}}},
    { "SCM Ethernet Combo",
      {2, {_pi("SCM"), _pi("Ethernet Combo card"), NULL, NULL}}},
    { "Sky Link Express",
      {2, {_pi("2408LAN"), _pi("Ethernet"), NULL, NULL}}},
    { "SMC EZCard-10 Ethernet",
      {2, {_pi("SMC"), _pi("EZCard-10-PCMCIA"), NULL, NULL}}},
    { "Socket EA Ethernet",
      {2, {_pi("Socket Communications Inc"),
           _pi("Socket EA PCMCIA LAN Adapter Revision D"), NULL, NULL}}},
    { "Socket EA Ethernet",
      {2, {_pi("Socket Communications Inc"),
           _pi("Socket EA PCMCIA LAN Adapter Revision E"), NULL, NULL}}},
    { "SuperSocket Ethernet",
      {2, {_pi("Telecom Device K.K."), _pi("SuperSocket RE450T"), NULL, NULL}}},
    { "Surecom Ethernet",
      {2, {_pi("TAMARACK"), _pi("Ethernet"), NULL, NULL}}},

    /* Fast Ethernet */
    { "Abocom LinkMate FE1000 Fast Ethernet",
      {2, {_pi("Fast Ethernet"), _pi("Adapter"), NULL, NULL}}},
    { "AnyCom ECO 10/100 Fast Ethernet",
      {2, {_pi("AnyCom"), _pi("ECO Ethernet 10/100"), NULL, NULL}}},
    { "COMPU-SHACK FASTline 10/100 Fast Ethernet",
      {2, {_pi("COMPU-SHACK"), _pi("FASTline PCMCIA 10/100 Fast-Ethernet"), NULL, NULL}}},
    { "Corega PCC-TX Fast Ethernet",
      {2, {_pi("corega K.K."), _pi("corega FastEther PCC-TX"), NULL, NULL}}},
    { "D-Link Fast Ethernet",
      {1, {_pi("ETHER-C16"), NULL, NULL, NULL}}},
    { "IO DATA PCET/TX Fast Ethernet",
      {2, {_pi("IO DATA"), _pi("PCETTX"), NULL, NULL}}},
    { "Laneed LD-10/100CD Fast Ethernet",
      {2, {_pi("Laneed"), _pi("LD-10/100CD"), NULL, NULL}}},
    { "Linksys EtherFast 10/100 Fast Ethernet",
      {2, {_pi("Linksys"), _pi("EtherFast 10/100 PC Card (PCMPC100)"), NULL, NULL}}},
    { "Logitec LPM-LN100TX Fast Ethernet",
      {2, {_pi("Logitec"), _pi("LPM-LN100TX"), NULL, NULL}}},
    { "Melco LPC2-TX Fast Ethernet",
      {2, {_pi("MELCO"), _pi("LPC2-TX"), NULL, NULL}}},
    { "Microcom C.E. Travel Card 10/100 Fast Ethernet",
      {2, {_pi("Microcom C.E."), _pi("Travel Card LAN 10/100"), NULL, NULL}}},
    { "NetGear FA410TXC Fast Ethernet",
      {2, {_pi("NETGEAR"), _pi("FA410TX"), NULL, NULL}}},
    { "NE2000 Compatible Fast Ethernet",
      {2, {_pi("PCMCIA"), _pi("10/100 Ethernet Card"), NULL, NULL}}},
    { "NE2000 Compatible Fast Ethernet",
      {2, {_pi("PCMCIA"), _pi("FAST ETHERNET CARD"), NULL, NULL}}},
    { "Planex FNW-3600T Fast Ethernet",
      {2, {_pi("Fast Ethernet"), _pi("Adapter"), NULL, NULL}}},
    { "Xircom CompactCard Ethernet",
      {2, {_pi("Xircom"), _pi("CompactCard Ethernet"), NULL, NULL}}}
};

manfid_info_t pcnet_cs_manfid[] = {
    { "Allied Telesis LA-PCM Ethernet",
      {0xc00f, 0x0002}},
    { "Connectware LANdingGear Ethernet",
      {0x0057, 0x1004}},
    { "KTI ETHER-C16 Fast ethernet",
      {0x0149, 0x0230}},
    { "Melco LPC2-T Ethernet",
      {0x01bf, 0x2216}},
    { "New Media LanSurfer",
      {0x0057, 0x0021}},
    { "NE2000 Compatible Ethernet",
      {0x0149, 0xc1ab}},
    { "Socket LP-E Ethernet",
      {0x0104, 0x000d}},
    { "Socket LP-E CF+ Ethernet",
      {0x0104, 0x0075}}
};

tuple_info_t pcnet_cs_tuple[] = {
    { "D-Link DE-650 Ethernet",
      {0x40, 0x0009, "D-Link PC Ethernet Card"}},
    { "Linksys Ethernet",
      {0x40, 0x0009, "E-CARD PC Ethernet Card"}}
};

#define PCNET_CS_VERS_SIZE	sizeof(pcnet_cs_vers)/sizeof(pcnet_cs_vers[0])
#define PCNET_CS_MANFID_SIZE	sizeof(pcnet_cs_manfid)/sizeof(pcnet_cs_manfid[0])
#define PCNET_CS_TUPLE_SIZE	sizeof(pcnet_cs_tuple)/sizeof(pcnet_cs_tuple[0])

/*====================================================================*/

/* ray_cs */
manfid_info_t ray_cs_manfid[] = {
    { "RayLink PC Card WLAN Adapter",
      {0x01a6, 0x0000}}
};

#define RAY_CS_VERS_SIZE	0
#define RAY_CS_MANFID_SIZE	sizeof(ray_cs_manfid)/sizeof(ray_cs_manfid[0])
#define RAY_CS_TUPLE_SIZE	0

/*====================================================================*/

/* smc91c92_cs */
vers_info_t smc91c92_cs_vers[] = {
    { "Megahertz CC10BT/2 Ethernet",
      {2, {_pi("Megahertz"), _pi("CC10BT/2"), NULL, NULL}}},
    { "New Media BASICS Ethernet",
      {2, {_pi("BASICS by New Media Corporation"), _pi("Ethernet"), _pi("SMC91C94"), NULL}}},
    { "Ositech Four of Diamonds Ethernet",
      {2, {_pi("Ositech"), _pi("Trumpcard:Four of Diamonds Ethernet"), NULL, NULL}}},
    { "SMC EtherEZ Ethernet",
      {2, {_pi("SMC"), _pi("EtherEZ Ethernet 8020"), NULL, NULL}}},
    { "Argosy EN220 Fast Ethernet",
      {2, {_pi("ARGOSY"), _pi("Fast Ethernet PCCard"), NULL, NULL}}},
    { "Dynalink L100C Fast Ethernet",
      {2, {_pi("DYNALINK"), _pi("L100C"), NULL, NULL}}},
    { "Farallon Enet",
      {2, {_pi("Farallon"), _pi("Farallon Enet"), NULL, NULL}}},
    { "Melco/SMC LPC-TX Fast Ethernet",
      {2, {_pi("MELCO/SMC"), _pi("LPC-TX"), NULL, NULL}}},
    { "Ositech Seven of Diamonds Fast Ethernet",
      {2, {_pi("Ositech"), _pi("Trumpcard:Seven of Diamonds Ethernet"), NULL, NULL}}}
};

#define SMC91C92_CS_VERS_SIZE	sizeof(smc91c92_cs_vers)/sizeof(smc91c92_cs_vers[0])
#define SMC91C92_CS_MANFID_SIZE	0
#define SMC91C92_CS_TUPLE_SIZE	0

/*====================================================================*/

/* wavelan_cs */
vers_info_t wavelan_cs_vers[] = {
    { "AT&T WaveLAN Adapter",
      {2, {_pi("AT&T"), _pi("WaveLAN/PCMCIA"), NULL, NULL}}},
    { "Digital RoamAbout/DS",
      {2, {_pi("Digital"), _pi("RoamAbout/DS"), NULL, NULL}}},
    { "Lucent Technologies WaveLAN Adapter",
      {2, {_pi("Lucent Technologies"), _pi("WaveLAN/PCMCIA"), NULL, NULL}}},
    { "NCR WaveLAN Adapter",
      {2, {_pi("NCR"), _pi("WaveLAN/PCMCIA"), NULL, NULL}}},
    { "NCR WaveLAN/IEEE Adapter",
      {2, {_pi("NCR"), _pi("WaveLAN/IEEE"), NULL, NULL}}}
};

#define WAVELAN_CS_VERS_SIZE	sizeof(wavelan_cs_vers)/sizeof(wavelan_cs_vers[0])
#define WAVELAN_CS_MANFID_SIZE	0
#define WAVELAN_CS_TUPLE_SIZE	0

/*====================================================================*/

/* xirc2ps_cs */
vers_info_t xirc2ps_cs_vers[] = {
    { "Compaq Ethernet",
      {2, {_pi("Compaq"), _pi("Ethernet LAN Card"), NULL, NULL}}},
    { "Compaq Netelligent 10/100 Ethernet",
      {2, {_pi("Compaq"), _pi("Netelligent 10/100 PC Card"), NULL, NULL}}},
    { "Xircom IIps Ethernet",
      {3, {_pi("Xircom"), _pi("*"), _pi("PS-CE2-10"), NULL}}},
    { "Xircom RE-10 Ethernet",
      {3, {_pi("Xircom"), _pi("*"), _pi("RE-10"), NULL}}},
    { "Intel EtherExpress PRO/100",
      {2, {_pi("Intel"), _pi("EtherExpress(TM) PRO/100 PC Card Mobile Adapter16"), NULL, NULL}}},
    { "Toshiba Advanced Network Fast Ethernet",
      {3, {_pi("Toshiba Information Systems"), _pi("*"), _pi("TPCENET"), NULL}}},
    { "Toshiba 10/100 Fast Ethernet",
      {2, {_pi("Toshiba"), _pi("10/100 Ethernet PC Card"), NULL, NULL}}},
    { "Xircom CE3-10/100 Fast Ethernet",
      {2, {_pi("Xircom"), _pi("*"), _pi("CE3-10/100"), NULL}}}
};

manfid_info_t xirc2ps_cs_manfid[] = {
    { "Accton Fast EtherCard-16",
      {0x01bf, 0x010a}}
};

#define XIRC2PS_CS_VERS_SIZE	sizeof(xirc2ps_cs_vers) / sizeof(xirc2ps_cs_vers[0])
#define XIRC2PS_CS_MANFID_SIZE	sizeof(xirc2ps_cs_manfid) / sizeof(xirc2ps_cs_manfid[0])
#define XIRC2PS_CS_TUPLE_SIZE	0

/*====================================================================*/

/* serial_cs */
vers_info_t serial_cs_vers[] = {
    { "Angia Fax/Modem",
      {2, {_pi("Intelligent"), _pi("ANGIA FAX/MODEM"), NULL, NULL}}},
    { "Cirrus Logic 14.4K Fax Modem",
      {2, {_pi("CIRRUS LOGIC"), _pi("FAX MODEM"), NULL, NULL}}},
    { "Compaq 28.8K Modem",
      {2, {_pi("COMPAQ"), _pi("PCMCIA 28800 FAX/DATA MODEM"), NULL, NULL}}},
    { "Compaq 33.6K Modem",
      {2, {_pi("COMPAQ"), _pi("PCMCIA 33600 FAX/DATA MODEM"), NULL, NULL}}},
    { "Dr. Neuhaus 14.4K FURY Card",
      {2, {_pi("Dr. Neuhaus"), _pi("FURY CARD 14K4"), NULL, NULL}}},
    { "E.TECH Bullet 33.6K Modem",
      {2, {_pi("PCMCIA   "), _pi("C336MX     "), NULL, NULL}}},
    { "Gateway 2000 TelePath Fax Modem",
      {4,{_pi("GATEWAY2000"), _pi("CC3144"), _pi("*"), _pi("PCMCIA MODEM")}}},
    { "Intel-compatible Modem",
      {2, {_pi("Intel"), _pi("MODEM 2400+"), NULL, NULL}}},
    { "Macronix Fax/Modem",
      {2, {_pi("MACRONIX"), _pi("FAX/MODEM"), NULL, NULL}}},
    { "Megahertz PCMCIA Modem",
      {4, {_pi("MEGAHERTZ"), _pi("*"), _pi("*"), _pi("PCMCIA MODEM")}}},
    { "Megahertz V.34 Modem",
      {3, {_pi("MEGAHERTZ"), _pi("*"), _pi("V.34 PCMCIA MODEM"), NULL}}},
    { "MultiTech MultiModem",
      {2, {_pi("Multi-Tech"), _pi("MT1432LT"), NULL, NULL}}},
    { "MultiTech V.34 Modem",
      {2, {_pi("Multi-Tech"), _pi("MT2834LT"), NULL, NULL}}},
    { "MultiTech MT5634ZLX",
      {2, {_pi("MultiTech"), _pi("PCMCIA 56K DataFax"), NULL, NULL}}},
    { "Toshiba T144PF Modem",
      {4, {_pi("TOSHIBA"), _pi("T144PF"), _pi("*"), _pi("PCMCIA MODEM")}}},
    { "Advantech COMpad-32/85",
      {3, {_pi("ADVANTECH"), _pi("COMpad-32/85"), _pi("1.0"), NULL}}},
    { "Advantech COMpad-32/85",
      {4, {_pi("ADV"), _pi("*"), _pi("TECH"), _pi("COMpad-32/85")}}},
    { "Argosy Dual Serial I/O",
      {2, {_pi("*"), _pi("RS-COM 2P"), NULL, NULL}}},
    { "Computerboards PCM-COM422",
      {2, {_pi("Computerboards, Inc."), _pi("PCM-COM422"), NULL, NULL}}},
    { "Fujitsu FMV-JMD711",
      {3, {_pi("FUJITSU"), _pi("FC14F "), _pi("MBH10213"), NULL}}},
    { "IOTech Dual RS-232",
      {2, {_pi("IOTech Inc "), _pi("PCMCIA Dual RS-232 Serial Port Card"), NULL, NULL}}},
    { "Quatech Dual RS-232",
      {2, {_pi("Quatech Inc"), _pi("PCMCIA Dual RS-232 Serial Port Card"), NULL, NULL}}}
};

manfid_info_t serial_cs_manfid[] = {
    { "National Instruments PCMCIA-232",
      {0x10b, 0x0d50}},
    { "National Instruments PCMCIA-232/2",
      {0x10b, 0x0d51}},
    { "National Instruments PCMCIA-485",
      {0x10b, 0x0d52}},
    { "National Instruments PCMCIA-485/2",
      {0x10b, 0x0d53}},
    { "National Instruments PCMCIA-232/4",
      {0x10b, 0xd180}},
    { "Omega Engineering QSP-100",
      {0x0137, 0x0025}},
    { "Quatech Quad RS-232",
      {0x0137, 0x001b}},
    { "Silicom 2S100 Dual Serial",
      {0x0089, 0x0301}},
    { "Socket Communications Dual RS-232",
      {0x0104, 0x0006}}
};

#define SERIAL_CS_VERS_SIZE	sizeof(serial_cs_vers)/sizeof(serial_cs_vers[0])
#define SERIAL_CS_MANFID_SIZE	sizeof(serial_cs_manfid)/sizeof(serial_cs_manfid[0])
#define SERIAL_CS_TUPLE_SIZE	0

/*====================================================================*/

/* Linked list of function definitions */
cardcfg_info_t root_card[] = {
    {VERS_1_IDENT, ide_cs_vers, IDE_CS_VERS_SIZE, &ide_cs},
    {MANFID_IDENT, ide_cs_manfid, IDE_CS_MANFID_SIZE, &ide_cs},
    {MANFID_IDENT, c574_cs_manfid, C574_CS_MANFID_SIZE, &c574_cs},
    {VERS_1_IDENT, c589_cs_vers, C589_CS_VERS_SIZE, &c589_cs},
    {MANFID_IDENT, c589_cs_manfid, C589_CS_MANFID_SIZE, &c589_cs},
    {MANFID_IDENT, aironet4500_cs_manfid, AIRONET4500_CS_MANFID_SIZE, &aironet4500_cs},
    {VERS_1_IDENT, fmvj18x_cs_vers, FMVJ18X_CS_VERS_SIZE, &fmvj18x_cs},
    {MANFID_IDENT, fmvj18x_cs_manfid, FMVJ18X_CS_MANFID_SIZE, &fmvj18x_cs},
    {VERS_1_IDENT, netwave_cs_vers, NETWAVE_CS_VERS_SIZE, &netwave_cs},
    {VERS_1_IDENT, nmclan_cs_vers, NMCLAN_CS_VERS_SIZE, &nmclan_cs},
    {VERS_1_IDENT, pcnet_cs_vers, PCNET_CS_VERS_SIZE, &pcnet_cs},
    {MANFID_IDENT, pcnet_cs_manfid, PCNET_CS_MANFID_SIZE, &pcnet_cs},
    {TUPLE_IDENT, pcnet_cs_tuple, PCNET_CS_TUPLE_SIZE, &pcnet_cs},
    {MANFID_IDENT, ray_cs_manfid, RAY_CS_MANFID_SIZE, &ray_cs},
    {VERS_1_IDENT, smc91c92_cs_vers, SMC91C92_CS_VERS_SIZE, &smc91c92_cs},
    {VERS_1_IDENT, wavelan_cs_vers, WAVELAN_CS_VERS_SIZE, &wavelan_cs},
    {VERS_1_IDENT, xirc2ps_cs_vers, XIRC2PS_CS_VERS_SIZE, &xirc2ps_cs},
    {MANFID_IDENT, xirc2ps_cs_manfid, XIRC2PS_CS_MANFID_SIZE, &xirc2ps_cs},
    {VERS_1_IDENT, serial_cs_vers, SERIAL_CS_VERS_SIZE, &serial_cs},
    {MANFID_IDENT, serial_cs_manfid, SERIAL_CS_MANFID_SIZE, &serial_cs},
    {0, NULL, 0, NULL}
};

#define ROOT_CARDS	sizeof(root_card) / sizeof(root_card[0])

/*====================================================================*/

funcid_info_t root_func[] = {
    { "Serial or Modem",
      {SERIAL_PORT},
      &serial_cs
    },
    { "ATA/IDE Fixed Disk",
      {FIXED_DISK},
      &ide_cs
    }
};

#define ROOT_FUNCS (sizeof(root_func) / sizeof(root_func[0]))
